==> pystaleds-0.1.4.tar.gz: pystaleds-0.1.4/src/ast_parsing.rs <==
use crate::parsing::extract_docstring;
use tree_sitter::{Node, Point};

/// Information about a function's signature and docstring.
pub(crate) struct FunctionInfo<'a, 'b> {
    pub(crate) params: &'b [(&'a str, Option<&'a str>)],
    pub(crate) docstring: Option<&'a str>,
    pub(crate) start_position: Point,
}

/// Extracts function information from a node if it is a function definition.
///
/// Uses a buffered params vector for performance, instead of allocating a new one
/// every time.
pub(crate) fn get_function_signature<'a, 'b>(
    node: &Node,
    source_code: &'a str,
    params: &'b mut Vec<(&'a str, Option<&'a str>)>,
) -> Option<FunctionInfo<'a, 'b>> {
    if !node.kind().eq("function_definition") {
        return None;
    }

    let params_node = node.child_by_field_name("parameters")?;
    params.clear();

    for child in params_node.children(&mut params_node.walk()) {
        let text = child
            .utf8_text(source_code.as_bytes())
            .expect("should be valid utf-8");

        if text == "self" {
            continue;
        }

        if child.kind() == "typed_parameter" || child.kind() == "typed_default_parameter" {
            let mut identifier = None;
            let mut typ = None;

            let mut d = child.walk();

            for inner_child in child.children(&mut d) {
                let text_of_inner_child = inner_child
                    .utf8_text(source_code.as_bytes())
                    .expect("should be valid utf-8");

                if inner_child.kind() == "identifier" {
                    identifier = Some(text_of_inner_child);
                } else if inner_child.kind() == "type" {
                    typ = Some(text_of_inner_child);
                }
            }

            if let (Some(identifier), Some(typ)) = (identifier, typ) {
                params.push((identifier, Some(typ)));
            }
        } else if child.kind() == "identifier" {
            params.push((text, None));
        } else if child.kind() == "default_parameter" {
            let (name, _) = text
                .split_once('=')
                .expect("parameter with default value should have '=' in the text");

            params.push((name, None));
        }
    }

    let mut block = None;
    for child in node.children(&mut node.walk()) {
        if child.kind() == "block" {
            block = Some(child);
            break;
        }
    }

    let block = block?;

    let content = block.utf8_text(source_code.as_bytes()).ok()?;
    let docstring = extract_docstring(content);

    let start_position = node.start_position();

    Some(FunctionInfo {
        params,
        docstring,
        start_position,
    })
}

==> pystaleds-0.1.5.tar.gz: pystaleds-0.1.5/src/ast_parsing.rs <==
use crate::parsing::extract_docstring;
use tree_sitter::{Node, Point};

/// Information about a function's signature and docstring.
pub(crate) struct FunctionInfo<'a, 'b> {
    pub(crate) params: &'b [(&'a str, Option<&'a str>)],
    pub(crate) docstring: Option<&'a str>,
    pub(crate) start_position: Point,
}

/// Extracts function information from a node if it is a function definition.
///
/// Uses a buffered params vector for performance, instead of allocating a new one
/// every time.
#[inline]
pub(crate) fn get_function_signature<'a, 'b>(
    node: &Node,
    source_code: &'a str,
    params: &'b mut Vec<(&'a str, Option<&'a str>)>,
) -> Option<FunctionInfo<'a, 'b>> {
    if !node.kind().eq("function_definition") {
        return None;
    }

    let params_node = node.child_by_field_name("parameters")?;
    params.clear();

    let mut cursor = params_node.walk();

    for child in params_node.children(&mut cursor) {
        let text = child
            .utf8_text(source_code.as_bytes())
            .expect("should be valid utf-8");

        if text == "self" {
            continue;
        }

        if child.kind() == "typed_parameter" || child.kind() == "typed_default_parameter" {
            let mut identifier = None;
            let mut typ = None;

            let mut d = child.walk();

            for inner_child in child.children(&mut d) {
                let text_of_inner_child = inner_child
                    .utf8_text(source_code.as_bytes())
                    .expect("should be valid utf-8");

                if inner_child.kind() == "identifier" {
                    identifier = Some(text_of_inner_child);
                } else if inner_child.kind() == "type" {
                    typ = Some(text_of_inner_child);
                }
            }

            if let (Some(identifier), Some(typ)) = (identifier, typ) {
                params.push((identifier, Some(typ)));
            }
        } else if child.kind() == "identifier" {
            params.push((text, None));
        } else if child.kind() == "default_parameter" {
            let (name, _) = text
                .split_once('=')
                .expect("parameter with default value should have '=' in the text");

            params.push((name, None));
        }
    }

    let block = node.children(&mut cursor).find(|c| c.kind() == "block")?;

    let content = block.utf8_text(source_code.as_bytes()).ok()?;
    let docstring = extract_docstring(content);

    let start_position = node.start_position();

    Some(FunctionInfo {
        params,
        docstring,
        start_position,
    })
}
